#! /usr/bin/env python
#
#    Client Base for PyWrite
#    Copyright (C) 2002  Michael Urman
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import pywconf
import os, sys, pwd, socket, select, time

class PyWriteClient:
    """PywriteServer is the base client class, providing a default client
    interface which acts like write(1), but stores logs in `logs_subdir`
    (defaults to ~/.pywrite/logs/)"""

    NONE = 0
    WRITING = 1

    def __init__(self, user):
        """Initialize the client.  Attempt to contact the server sockets for 
        exists, attempt to verify it is alive.  If not, wipe and attempt to
        allocate the socket."""

        # ensure pywrite_subdir exists and is a directory
        if not pywconf.checkdirs():
            sys.exit(-1)

        self.address = pywconf.socket(user)
        serv = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)

        if not os.path.exists(pywconf.subdir(user)):
            print user, "appears to not use pywrite.  Falling back to write"
            self.write(sys.argv)

        try:
            serv.connect(self.address)
        except socket.error, connect_error:
            print user, "is not running pywrited.  Falling back to write"
            self.write(sys.argv)

        self.server_socket = serv
        self.status = PyWriteClient.NONE
        self.user = user

    def write(self, args):
        """write(self, args) -> None

        Fallback to system write with arguments args"""

        for path in os.defpath.split(':'):
            binwrite = os.path.join(path, 'write')
            if os.path.isfile(binwrite) and os.access(binwrite, os.X_OK):
                break
        else:
            print >> sys.stderr, "Error: Cannot find write.  Check your path!"
            sys.exit(-1)

        os.execl(binwrite, *args)

    def __del__(self):
        self.server_socket.shutdown(2)
        self.server_socket.close()

    def run(self):
        user = self.user
        try:
            while 1:
                text = raw_input()

                try:
                    self.server_socket.sendall(text+'\n', socket.MSG_DONTWAIT)
                except socket.error:
                    print >> sys.stderr, user, "stopped accepting pywrites."
                    raise EOFError

                if self.status != PyWriteClient.WRITING:
                    self.status = PyWriteClient.WRITING

                    timestr = time.ctime(time.time())

                    self.filename = os.path.join(pywconf.logdir(), user)
                    self.file = open(self.filename, 'a')
                    print >> self.file, '-', "\tMessage to %s at %s..." % (user, timestr)
                print >> self.file, '-', text
                self.file.flush()

        except (EOFError, KeyboardInterrupt):
            if self.status == PyWriteClient.WRITING:
                self.status = PyWriteClient.NONE
                print >> self.file, '-', "EOF"
